@@ -4,32 +4,44 @@ import json
 import re
 from datetime import date, datetime
 from os import walk
-from os.path import getsize, splitdrive, splitext
-from pathlib import Path
+from os.path import (
+    exists,
+    expanduser,
+    expandvars,
+    getsize,
+    splitdrive,
+    splitext,
+)
+from pathlib import Path, PurePath
 from typing import Any, Callable, Dict, Generator, List, Optional, Tuple, Union
 from unicodedata import normalize
 
 import requests_cache
 from requests.adapters import HTTPAdapter
 
-from mnamer.const import CACHE_PATH, CURRENT_YEAR
+from mnamer.const import CACHE_PATH, CURRENT_YEAR, SUBTITLE_CONTAINERS
 
 __all__ = [
     "clean_dict",
     "clear_cache",
     "crawl_in",
     "crawl_out",
+    "filename_replace",
     "filter_blacklist",
-    "filter_extensions",
+    "filter_containers",
     "findall",
     "fn_chain",
     "fn_pipe",
     "format_dict",
+    "format_exception",
     "format_iter",
+    "get_filesize",
     "get_session",
+    "is_subtitle",
     "json_dumps",
-    "normalize_extension",
-    "normalize_extensions",
+    "json_loads",
+    "normalize_container",
+    "normalize_containers",
     "parse_date",
     "request_json",
     "str_fix_padding",
@@ -92,9 +104,9 @@ def crawl_out(filename: str) -> Optional[Path]:
 
 def filename_replace(filename: str, replacements: Dict[str, str]) -> str:
     """Replaces keys in replacements dict with their values."""
-    base, ext = splitext(filename)
+    base, container = splitext(filename)
     base = str_replace(base, replacements)
-    return base + ext
+    return base + container
 
 
 def filter_blacklist(paths: List[Path], blacklist: List[str]) -> List[Path]:
@@ -110,15 +122,15 @@ def filter_blacklist(paths: List[Path], blacklist: List[str]) -> List[Path]:
     ]
 
 
-def filter_extensions(
-    file_paths: List[Path], valid_extensions: List[str]
+def filter_containers(
+    file_paths: List[Path], valid_containers: List[str]
 ) -> List[Path]:
-    """Filters (set intersection) a collection of extensions."""
-    valid_extensions = normalize_extensions(valid_extensions)
+    """Filters (set intersection) a collection of containers."""
+    valid_containers = normalize_containers(valid_containers)
     return [
         file_path
         for file_path in file_paths
-        if not valid_extensions or file_path.suffix.lower() in valid_extensions
+        if not valid_containers or file_path.suffix.lower() in valid_containers
     ]
 
 
@@ -155,6 +167,10 @@ def format_dict(body: Dict[Any, Any]) -> str:
     )
 
 
+def format_exception(body: Exception) -> str:
+    return str(body)
+
+
 def format_iter(body: list) -> str:
     """
     Formats an iterable into a multi-line bulleted string of its values.
@@ -162,6 +178,10 @@ def format_iter(body: list) -> str:
     return "\n".join(sorted([f" - {getattr(v, 'value', v)}" for v in body]))
 
 
+def is_subtitle(container: Optional[str]) -> bool:
+    return bool(container) and container.endswith(tuple(SUBTITLE_CONTAINERS))
+
+
 def get_session() -> requests_cache.CachedSession:
     """Convenience function that returns request-cache session singleton."""
     if not hasattr(get_session, "session"):
@@ -174,7 +194,7 @@ def get_session() -> requests_cache.CachedSession:
     return get_session.session
 
 
-def get_filesize(path: Path) -> str:
+def get_filesize(path: Union[PurePath, Path]) -> str:
     """Returns the human-readable filesize for a given path."""
     size = getsize(path)
     for unit in ["B", "KB", "MB", "GB", "TB"]:
@@ -197,17 +217,27 @@ def json_dumps(d: Dict[str, Any]) -> str:
     )
 
 
-def normalize_extension(extension: str) -> str:
-    """Ensures all extensions begin with a dot."""
-    assert extension
-    if extension and extension[0] != ".":
-        extension = f".{extension}"
-    return extension.lower()
+def json_loads(path: str) -> Dict[str, Any]:
+    json_data = ""
+    path = expanduser(path)
+    path = expandvars(path)
+    if exists(path):
+        with open(path, mode="r") as fp:
+            json_data = fp.read()
+    return json.loads(json_data) if json_data else {}
 
 
-def normalize_extensions(extension_list: List[str]) -> List[str]:
-    """For a list of extensions ensures that all extensions begin with a dot."""
-    return [normalize_extension(extension) for extension in extension_list]
+def normalize_container(container: str) -> str:
+    """Ensures all containers begin with a dot."""
+    assert container
+    if container and container[0] != ".":
+        container = f".{container}"
+    return container.lower()
+
+
+def normalize_containers(container_list: List[str]) -> List[str]:
+    """For a list of containers ensures that all containers begin with a dot."""
+    return [normalize_container(container) for container in container_list]
 
 
 def parse_date(value: Union[str, date, datetime]) -> date:
@@ -305,11 +335,15 @@ def str_replace_slashes(s: str) -> str:
 
 def str_sanitize(filename: str) -> str:
     """Removes illegal filename characters and condenses whitespace."""
-    base, ext = splitext(filename)
+    base, container = splitext(filename)
+    if is_subtitle(container):
+        base = base.rstrip(".")
+        base, container_prefix = splitext(base)
+        container = container_prefix + container
     base = re.sub(r"\s+", " ", base)
     drive, tail = splitdrive(base)
     tail = re.sub(r'[<>:"|?*&%=+@#`^]', "", tail)
-    return drive + tail.strip("-., ") + ext
+    return drive + tail.strip("-., ") + container
 
 
 def str_scenify(filename: str) -> str:
@@ -489,7 +523,7 @@ def year_range_parse(
     default_start = 1900
     default_end = CURRENT_YEAR
     try:
-        start, dash, end = re.match(regex, str(years)).groups()
+        start, dash, end = re.match(regex, str(years).strip()).groups()
     except AttributeError:
         start, end, dash = None, None, True
     if not start and not end:
